/**************************************************************************
 *  Weather, version 2.0 - generates random weather                       *
 *                                                                        *
 *  Copyright 1988, 1989, 1999 by Brett Slocum.  All rights reserved.     *
 *  Permission is granted to distribute or modify this program, or use    *
 *  portions of this program in other programs, as long as this notice    *
 *  remains intact.                                                       *
 *  This program or its derivatives may not be sold for profit without    *
 *  permission of the author.                                             *
 *                                                                        *
 *  To run:   <region name> [<start year> <end year>]                     *
 *                                                                        *
 *      If the start and end years are not supplied, the program          *
 *  produces one year of weather.                                         *
 *                                                                        *
 *  Example:  england 1066 1075                                           *
 *            will produce 10 years of weather starting in 1066 and       *
 *            in 1075.                                                    *
 *                                                                        *
 *  Original UNIX Version: Brett Slocum <slocum@skypoint.com>             *
 *                                                                        *
 *  Version 2.0 changes:                                                  *
 *      Converted from K&R C to C++.                                      *
 *                                                                        *
 *  Bugs reported and modifications suggested by:                         *
 *      "251 degrees in a Minnesota winter" bug:                          *
 *          Michael Kent <...!{uflorida, uunet}!aleytys!mike>             *
 *          Mark E Hunter <meh@wucs1.wustl.edu>                           *
 *          Dan Farmer <ajm@mentor.cc.purdue.edu>                         *
 *          Kevin W. Thomas <uokmax!kwthomas@uokmax.ecn.uoknor.edu>       *
 *                                                                        *
 *      VMS and Shire calendar modifications:                             *
 *          Karsten Spang, NBI, Copenhagen <SPANG@nbivax.nbi.dk>          *
 *                                                                        *
 *      MS-DOS Microsoft C changes:                                       *
 *          Jonathan Bayer <uunet!ispi!jbayer>                            *
 *                                                                        *
 *      MS-DOS Microsoft C changes:                                       *
 *          Jonathan Bayer <uunet!ispi!jbayer>                            *
 *                                                                        *
 **************************************************************************/

// WeatherApp.cpp : Defines the class behaviors for the application.
//

#include <cstdlib>
#include <cstring>
#include <iostream.h>
#include <iomanip.h>

#include "WeatherApp.h"
#include "Calendar.h"
#include "Global.h"
#include "Climate.h"
#include "WeatherReport.h"

using namespace std;

void main(int argc, char *argv[])  
{
     int nStartYear = 1,         /* starting year */
         nEndYear = 1;           /* ending year */

	 for (int i = 1; i < argc; i++)
	 {
		 string strArg = argv[i];
		 if (strArg.length() > 2)
		 {
			 int nPos = strArg.find_first_not_of("/-");
			 nPos++;
			 char sData[32];
			 int nLength = strArg.length();
			 strArg.copy(sData, nLength-nPos, nPos);
			 sData[nLength-nPos] = 0;
			 string strData = sData;
			 char cCmd = strArg[1];

			 if ((strArg[0] == '-') || (strArg[0] == '/'))
			 {
				 switch (cCmd)
				 {
				 case 'S': // start year
				 case 's':
					 nStartYear = atoi(strData.c_str());
					 break;
				 case 'E': // end year
				 case 'e':
					 nEndYear = atoi(strData.c_str());
					 break;
				 case 'C': // calendar
				 case 'c':
					 if ((strData.compare("gregorian") == 0) || (strData.compare("GREGORIAN") == 0))
					 {
						 CGlobal::setCalendar(CCalendar::GREGORIAN);
					 }
					 else if ((strData.compare("japan") == 0) || (strData.compare("JAPAN") == 0))
					 {
						 CGlobal::setCalendar(CCalendar::JAPAN);
					 }
					 else if ((strData.compare("middle_earth") == 0) || (strData.compare("MIDDLE_EARTH") == 0))
					 {
						 CGlobal::setCalendar(CCalendar::MIDDLE_EARTH);
					 }
					 else if ((strData.compare("tekumel") == 0) || (strData.compare("TEKUMEL") == 0))
					 {
						 CGlobal::setCalendar(CCalendar::TEKUMEL);
					 }
					 break;
				 case 'W': // weather/climate
				 case 'w':
					 if ((strData.compare("n_atlantic") == 0) || (strData.compare("N_ATLANTIC") == 0))
					 {
						 CGlobal::setClimate(CClimate::N_ATLANTIC);
					 }
					 else if ((strData.compare("mid_atlantic") == 0) || (strData.compare("MID_ATLANTIC") == 0))
					 {
						 CGlobal::setClimate(CClimate::MID_ATLANTIC);
					 }
					 else if ((strData.compare("s_atlantic") == 0) || (strData.compare("S_ATLANTIC") == 0))
					 {
						 CGlobal::setClimate(CClimate::S_ATLANTIC);
					 }
					 else if ((strData.compare("n_pacific") == 0) || (strData.compare("N_PACIFIC") == 0))
					 {
						 CGlobal::setClimate(CClimate::N_PACIFIC);
					 }
					 else if ((strData.compare("gulf") == 0) || (strData.compare("GULF") == 0))
					 {
						 CGlobal::setClimate(CClimate::GULF);
					 }
					 break;
				 default:
					 cout << "Usage: " << argv[0] << " [-{s|S}<start year>] [-{e|E}<end year>] [-{c|C}<calendar type>] [-{w|W}<climate type>]" << endl;
					 cout << "   where <calendar type> = {gregorian|GREGORIAN|japan|JAPAN|middle_earth|MIDDLE_EARTH|tekumel|TEKUMEL}" << endl;
					 cout << "   and   <climate type> = {n_atlantic|N_ATLANTIC|mid_atlantic|MID_ATLANTIC|s_atlantic|S_ATLANTIC|n_pacific|N_PACIFIC|gulf|GULF}" << endl;
					 return;
					 break;
				 }
			 }
		 }
	 }

	 CWeatherReport* pReport = CWeatherReport::instance();
	 CCalendar* pCalendar = pReport->getCalendar();

	 // initialize for winter to set snow depth properly
	 pReport->initWinter();

     for (int nYear = nStartYear; nYear <= nEndYear; nYear++)
     {
		 // print weather for a year
         pReport->init();
		 for (int nMonth = pCalendar->getFirstMonth(); nMonth <= pCalendar->getLastMonth(); nMonth++)
		 {
			 for (int nDay = 1; nDay <= pCalendar->getMonthLength(nMonth); nDay ++)
			 {
				 pReport->generate(nDay, nMonth, nYear);
				 pReport->print();
			 }
		 }
     }
}


